/***************************************************************************
                          cuser.cpp  -  description
                             -------------------
    begin                : Thu May 8 2003
    copyright            : (C) 2003 by Daniel Muller
    email                : dan at verliba dot cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cuser.h"

namespace nDirectConnect
{

cTime user_global_time;

cUser::cUser() :
	cObj((const char *)"User"),
	mShare(0),
	mFloodPM(0.00,30.,10,user_global_time.Get())
{
	mxConn = NULL;
	mxServer = NULL;
	SetRight( eUR_CHAT,0, true );
	SetRight( eUR_SEARCH, 0, true );
	SetRight( eUR_CTM, 0, true );
	SetRight( eUR_KICK, 0, false );
	SetRight( eUR_DROP, 0, false );
	SetRight( eUR_TBAN, 0, false );
	SetRight( eUR_PBAN, 0, false );
	mProtectFrom = 0;
	mHideKick = false;
	IsPassive = false;
	memset(mFloodHashes, 0 ,sizeof(mFloodHashes));
}

/** Constructor */
cUser::cUser(const string &nick) :
	mNick( nick ),
	mxConn(NULL),
	mxServer(NULL),
	mShare(0),
	mHideKicksForClass(eUC_NORMUSER),
	mFloodPM(0.00,30.,30,user_global_time.Get())
{
	SetClassName("cUser");
	mInList = false;
	IsPassive = false;
	mClass = eUC_NORMUSER;
	mRights = 0;
	mToBan = false;
	mVisibleClassMin = eUC_NORMUSER;
	mOpClassMin = eUC_NORMUSER;
	SetRight( eUR_CHAT, 0, true );
	SetRight( eUR_SEARCH, 0, true );
	SetRight( eUR_CTM, 0, true );
	SetRight( eUR_KICK, 0, false );
	SetRight( eUR_DROP, 0, false );
	SetRight( eUR_TBAN, 0, false );
	SetRight( eUR_PBAN, 0, false );
	mProtectFrom = 0;
	mHideKick = false;
	memset(mFloodHashes, 0 , sizeof(mFloodHashes));
	//@todo HideKick is to be replaced by usage HideKickForCLass
}

cUser::~cUser()
{
}


/** return tru if user needs a password and the password is correct */
bool cUser::CheckPwd(const string &pwd)
{
	if( !mxConn || !mxConn->mRegInfo ) return false;
	return mxConn->mRegInfo->PWVerify(pwd);
}

/** perform a registration: set class, rights etc... precondition: password was al right */
void cUser::Register()
{
	if (!mxConn || !mxConn->mRegInfo) return;
	if (mxConn->mRegInfo->mPwdChange) return;
	mClass = (tUserCl)mxConn->mRegInfo->mClass;
	mProtectFrom = mxConn->mRegInfo->mClassProtect;
	mHideKicksForClass = mxConn->mRegInfo->mClassHideKick;
	mHideKick = mxConn->mRegInfo->mHideKick;
}

};

/*!
    \fn nDirectConnect::cUser::ShareEnthropy()
 */
long nDirectConnect::cUser::ShareEnthropy(const string &sharesize)
{
	char diff[20];
	int count[20];

	int i,j;
	long score = 0;
	// calculate counts of every byte in the sharesize string
	for( i = 0; i < sharesize.size(); i++ )
	{
		count[i] = 0;
		for( j = i+1; j < sharesize.size(); j++ )
			if(sharesize[i]==sharesize[j]) ++ count[i];
	}

	// make the weighted sum
	for( i = 0; i < sharesize.size(); ) score += count[i]*++i;

	// calculate the differences
	for( i = 0; i < sharesize.size()-1; i++ )
		diff[i] = 10 + sharesize[i-1] - sharesize[i];

	// calculate counts of every byte in the sharesize string differences
	for( i = 0; i < sharesize.size()-1; i++ )
	{
		count[i] = 0;
		for( j = i+1; j < sharesize.size(); j++ )
			if(diff[i]==diff[j]) ++ count[i];
	}
	for( i = 0; i < sharesize.size(); ) score += count[i]*++i;

	return score;
}



/*!
    \fn nDirectConnect::cUser::DisplayInfo(ostream &os, int DisplClass)
 */
 void nDirectConnect::cUser::DisplayInfo(ostream &os, int DisplClass)
{ 
	static const char *ClassName[]={"Guest","Registred", "VIP", "Operator", "Cheef", "Admin" , "6-err","7-err", "8-err","9-err","Master"};
	os << "Nick: " << mNick << "\r\n";
	os << "Class: " << ClassName[this->mClass] << " (" << mClass << ")" << "\r\n";
	if(DisplClass >= eUC_CHEEF) os << "InList: " << this->mInList;
	if( !this->mxConn)
	{
		os << "Special User" << "\r\n";
	}
	else
	{
		if(DisplClass >= eUC_OPERATOR) os << "IP: " << mxConn->AddrIP() << "\r\n";
		if(DisplClass >= eUC_OPERATOR && mxConn->AddrHost().size()) os << "Host: " << mxConn->AddrHost() << "\r\n";
		if( this->mClass != this->mxConn->GetTheoricalClass() )
			os << "Default class: " << this->mxConn->GetTheoricalClass() << "\r\n";
		if(mxConn->mCC.size()) os << "Country Code: " << mxConn->mCC << "\r\n";
		if(mxConn->mRegInfo != NULL) os << "\r\nReg Information:\r\n" << *(mxConn->mRegInfo) <<"\r\n\r\n";
	}
}


/*!
    \fn nDirectConnect::cUser::ApplyRights(cPenaltyList::sPenalty &pen)
 */
void nDirectConnect::cUser::ApplyRights(cPenaltyList::sPenalty &pen)
{
	mGag = pen.mStartChat;
	mNoSearch = pen.mStartSearch;
	mNoCTM = pen.mStartCTM;
	mCanKick = pen.mStopKick;
}

bool nDirectConnect::cChatRoom::ReceiveMsg(cConnDC *conn, cMessageDC *msg)
{
	ostringstream os;
	if (msg->mType == eDC_TO)
	{
		if(conn && conn->mpUser && mCol)
		{
			string omsg;
			string start, end;

			if (mCol->ContainsNick(conn->mpUser->mNick))
			{

					//this is a standard TO/From message, the lastBCNick is used so that the sender
					//does not have to receive the message also
					mS->mP.Create_PMForBroadcast(start, end, mNick, conn->mpUser->mNick, msg->ChunkString(eCH_PM_MSG));
					bool temp_save_inlist = conn->mpUser->mInList;
					conn->mpUser->mInList = false;
					mCol->SendToAllWithNick(start,end);
					conn->mpUser->mInList = temp_save_inlist;
				}
				else
				{
					os << "You can't use " << mNick << " rather use main chat ;o)..";
					mS->DCPrivateHS(os.str(),conn, &mNick);
				}
		}
	}
	return true;
}

bool nDirectConnect::cMainRobot::ReceiveMsg(cConnDC *conn, cMessageDC *message)
{
	ostringstream os;
	if (message->mType == eDC_TO)
	{
		string &msg = message->ChunkString(eCH_PM_MSG);
		if (!mS->mP.ParseForCommands(msg, conn))
		{
			cUser *other = mS->mUserList.GetUserByNick ( mS->LastBCNick );
				if(other && other->mxConn)
				{
					mS->DCPrivateHS(message->ChunkString(eCH_PM_MSG),other->mxConn,&conn->mpUser->mNick);
				}
				else
					mS->DCPrivateHS("Hub-security doesn't accept pm's, but you can try a +command (or !)", conn);
		}
	}
   return true;
}

nDirectConnect::cPluginRobot::cPluginRobot(const string &nick, cVHPlugin *pi, cServerDC *server) :
	cUserRobot(nick, server), mPlugin(pi)
{}

bool nDirectConnect::cPluginRobot::ReceiveMsg(cConnDC *conn, cMessageDC *message)
{
	ostringstream os;
	if (message->mType == eDC_TO)
	{
		mPlugin->RobotOnPM( this, message, conn);
	}
	return true;
}
